/* FGenIOtest.c

Port A: Data bus - input and output - NOT used as External I/O

Port B:
	0,1: reserve for SPI A,B clocks
	2: LCD_RS: Low = Command,  Data = High
   3: LCD_ENA: high pulse
	4: Latch Enable for LEDs: pulse high
	5,6: not used
   7: Output enable for reading switches: high to enable

Port C:
	3: SPI TxC data

Port D:
	0: Quadrature Decoder - I
	1: Quadrature Decoder - Q
	2: Output Enable for LEDs: 0 = enable
	3: not used

Port E:
	0: PPM-0
	1: PPM-1
	2: PPM-2
	3: PPM-3
   5: PWM-0
   6: PWM-1
   7: SPI clock C
*/

#define TEST_PORT_SW		// A, B, D, E, IC3, SW

main ()
{	int i, j, k;
	char Switches[9];

// initial set up of parallel ports
	WrPortI ( PADR, NULL, 0 );				// Port A = 0
	WrPortI ( SPCR, NULL, 0x84 );			// Port A = output

	WrPortI ( PBDR, &PBDRShadow, 0 );		// Port B = 0
   WrPortI ( PBDDR, &PBDDRShadow, 0xFF ); // Port B = output

	WrPortI ( PDDR, &PDDRShadow, 0 );		// Port D = 0
	WrPortI ( PDDDR, &PDDDRShadow, PDDDRShadow|0x0C ); // PD2,3 = output
   WrPortI ( PDDDR, &PDDDRShadow, PDDDRShadow&~0x03 ); // PD0,1 = input

	WrPortI ( PEDR, &PEDRShadow, 0 );		// Port E = 0
   WrPortI ( PEDDR, &PEDDRShadow, PEDDRShadow|~0x10 ); // PE0..3,5..7 = output

#ifdef TEST_PORT_A	// test port A outputs
	while (1)
   {	for (i=0; i<=7; i ++ )
		{	if ( i==0 ) j=1;					// start with bit 0
      	WrPortI ( PADR, NULL, j );		// write it
         j <<= 1;								// next bit
      }
   }
#endif


#ifdef TEST_PORT_B	// test port B outputs
	while (1)
   {	for (i=0; i<=7; i ++ )
		{	if ( i==0 ) j=1;					// start with bit 0
      	WrPortI ( PBDR, NULL, j );		// write it
         j <<= 1;								// next bit
      }
   }
#endif


#ifdef TEST_PORT_D	// test port D outputs
	while (1)
   {	WrPortI ( PDDR, NULL, 4 );			// bit 2
		WrPortI ( PDDR, NULL, 8 );			// bit 3
		WrPortI ( PDDR, NULL, 0 );
   }
#endif


#ifdef TEST_PORT_E	// test port E outputs
	while (1)
   {	for (i=0; i<=7; i ++ )
		{	if ( i==0 ) j=1;					// start with bit 0
			if ( i==4 )							// skip bit 4
         {	i = 5;
         	j <<= 1;
         }
      	WrPortI ( PEDR, NULL, j );		// write it
         j <<= 1;								// next bit
      }
   }
#endif


#ifdef TEST_PORT_IC3		// test the outputs of IC3 - bits 0..3
	BitWrPortI ( PDDR, &PDDRShadow, 1, 2 ); // /OE disable
	while (1)
   {	for (i=0; i<=3; i ++ )
		{	if ( i==0 ) j=1;					// start with bit 0
      	WrPortI ( PADR, NULL, j );		// write it
         j <<= 1;								// next bit
         BitWrPortI ( PBDR, &PBDRShadow, 1, 4 );
         BitWrPortI ( PBDR, &PBDRShadow, 0, 4 );
         BitWrPortI ( PDDR, &PDDRShadow, 0, 2 ); // /OE enable
      }
   }
#endif


#ifdef TEST_PORT_SW		// test the switches

	WrPortI ( SPCR, NULL, 0x80 );		// port A = input
   BitWrPortI ( PBDR, &PBDRShadow, 1, 7 );
   printf ( "     76543210\n\r" );
	while (1)
   {
	   k = RdPortI ( PADR );
 		for (i=0; i<=7; i++)
      {	if ( k & (1<<i) ) Switches[7-i] = '1';
      	else Switches[7-i] = '0';
      }
		Switches[8] = 0;
      printf ( "SWs: %s\r", Switches );
   }
#endif

}