// Fser_test.C

#define FSER_DEBUG debug
#define DAC_DEBUG	debug

#define	DOUT_LE_PORT	B
#define	DOUT_LE_BIT		4
#define	DOUT_OE_PORT	D
#define	DOUT_OE_BIT		2

#define	DAC_CS_REG		B			// chip select register
#define	DAC_CS_BIT		5			// PB5 = DAC /CS

#define	DDS_CS_REG		B			// chip select register
#define	DDS_CS_BIT		6			// PB6 = DDS /CS

#define	PB_MASK		0x60			// PB5,6 = high

#use MCP4822.lib
#use AD9834.lib

#define DDS_AMP_DAC	DAC_A
#define DDS_OFF_DAC	DAC_B

#define	FSERC_ENABLE	// use serial port C for SPI
#use fser_4.lib			// used by both AD9834 and MCP4822 libraries

#define	SPIport			FSER_C
#define	SPIportW			FSER_C|0x8000
#define	SPIclk			FSER_RXC
#define	FSERC_SYNC 		SPI_MODE_MASTER
#define	FSERC_CLK		PEDR

#use HD44780.Lib					// for MsDelay


void InitIO ( void );

main ()
{	float Volts;

	InitIO();									// MUST preceede LCDinit

   AD9834init();

/*  DAC tests
	while (1)
   {
		for ( Volts = 0; Volts < 2.048; Volts += .1 )
      {
			DAC_volts ( DAC_A, Volts );
			DAC_volts ( DAC_B, 2.047-Volts );
	      MsDelay (1);
      }

		DAC_raw ( DAC_A, 4095 );
		DAC_raw ( DAC_B, 2048 );
      MsDelay (1);
		DAC_raw ( DAC_A, 0 );
		DAC_raw ( DAC_B, 0 );
      MsDelay (1);
*/

}


/*
Port A: Data bus - input and output - NOT External I/O

Port B:
	0,1: reserve for SPI A,B clocks
	2: LCD_RS: Low = Command,  Data = High
   3: LCD_ENA: high pulse
	4: Latch Enable for LEDs: pulse high
	5: DAC /CS
   6: DDS /CS
   7: Output enable for reading switches: high to enable

Port C:
	3: SPI TxC data

Port D:
	0: Quadrature Decoder - I
	1: Quadrature Decoder - Q
	2: Output Enable for LEDs: 0 = enable
	3: not used

Port E:
	0: PPM-0
	1: PPM-1
	2: PPM-2
	3: PPM-3
   5: PWM-0
   6: PWM-1
   7: SPI clock C
*/

void InitIO ( void )
{	int i, j;

   WrPortI ( PWL0R, &PWL0RShadow, 0 );	// disable PWM interrupts
	WrPortI ( TAT1R, &TAT1RShadow, 0 );	// TA1 = /1
   WrPortI ( TCCR, &TCCRShadow, 4 );	// disable interrupts, use TA1

// initial set up of parallel ports

//Port A: Data bus - input and output - NOT External I/O
	WrPortI ( PADR, NULL, 0 );				// Port A = 0
	WrPortI ( SPCR, NULL, 0x84 );			// Port A = output

//Port B:
//	0,1: reserve for SPI A,B clocks
//	2: LCD_RS: Low = Command,  Data = High
// 3: LCD_ENA: high pulse
//	4: Latch Enable for LEDs: pulse high
//	5: DAC /CS
// 6: DDS /CS
// 7: Output enable for reading switches: high to enable
	WrPortI ( PBDR, &PBDRShadow, PB_MASK );
   WrPortI ( PBDDR, &PBDDRShadow, 0xFF ); // Port B = output

//Port C:
// 3: SPI TxC data
	WrPortI ( PCALR, &PCALRShadow, PCALRShadow &~0xC0 ); // PC3 = TxC = alt out 0
	BitWrPortI ( PCFR, &PCFRShadow, 1, 3 ); // enable alt function on PC3
	BitWrPortI ( PCDDR, &PCDDRShadow, 1, 3 ); // enable PC3 as output

//Port D:
//	0: Quadrature Decoder - I
//	1: Quadrature Decoder - Q
//	2: Output Enable for LEDs: 0 = enable
//	3: not used
	WrPortI ( PDDR, &PDDRShadow, 0 );	// Port D = 0
   WrPortI ( PDDDR, &PDDDRShadow, PDDDRShadow&~0x03 ); // PD0,1 = QuadDecoder
	WrPortI ( PDDDR, &PDDDRShadow, PDDDRShadow|0x0C ); // PD2,3 = output

//Port E:
//	0: PPM-0
//	1: PPM-1
//	2: PPM-2
//	3: PPM-3
// 5: PWM-0
// 6: PWM-1
// 7: SPI clock C
   PEAHRShadow &= 0x03;						// leave bits for PE4 alone
	PEAHRShadow |= (0x08 | 0x20 | 0xC0); // PE5,6,7 = alt functions
	WrPortI ( PEAHR, &PEAHRShadow, PEAHRShadow );	// bits 7..4
	WrPortI ( PEALR, &PEALRShadow, 0xAA );	// bits 3..0 = alt function 2
	WrPortI ( PEFR,  &PEFRShadow,  PEFRShadow|~0x10 ); // leave bit 4 alone, rest are alt output
	WrPortI ( PEDDR, &PEDDRShadow, PEDDRShadow|~0x10 ); // leave bit 4 alone, rest are output

// Output latch: IC3
   BitWrPortI ( PBDR, &PBDRShadow, 1, DOUT_LE_BIT ); // pulse
   BitWrPortI ( PBDR, &PBDRShadow, 0, DOUT_LE_BIT ); //		LE
	BitWrPortI ( PDDR, &PDDRShadow, 0, DOUT_OE_BIT ); // enable the outputs

	WrPortI ( TCCSR, NULL, 1 );			// enable Timer C clock
   WrPortI ( TAPR, &TAPRShadow, 1 );	// set TAPR to /2

   fSerOpen ( SPIport, 100000L, 1, FSER_RXC );

} // InitIO

