/* FunctionGenerator.C

Need to fix:
	DDS phase * 2
   save settings did not ask for name


This program uses the following custom libraries:
Menu.Lib					generic menu for use on a 4x20 character display (LCD)
HD44780.Lib				hardware-specific display library
MCP4822.Lib				hardware-specific for MCP4822 dual DAC
AD9834.Lib				hardware-specific for AD9834 DDS
fser_4.Lib				fast serial library supports both sync and async

FGen_IO_defs.C			contains hardware-specific definitions and the
								hardware initialization function.

Standard Dynamic C libraries:
IDBLOCK_API.LIB		access User Blocks for saving & retrieving configuration
qd.lib					only if USE_QUAD_ENCODER is defined

In theory, the only file that needs to be edited for hardware changes is
	FGen_IO_defs.C
except for Arb_Set_State which has some port specific functions

See the comments in Menu.Lib for details on what functions the display library
must support.

ARB notes -------------------------------------------------------------
A file is imported into the program from the C drive: C:\ARBdata.txt.
This file contains data for the ARB feature.  The file MUST have the
following format.  There is no error detection!
	Line 1:
	 	byte 0:	A
   	bytes 1-3: byte count of data (max value = 255)
	Lines 2-n: data bytes in two digit Hex ASCII format
   	single space between bytes
      multiple lines may be used
example:
A6
AA F0 AA F0 00 00
-----------------------------------------------------------------------

IMPORTANT: time values are always in microseconds

NOTE: the reason for the "+1" in the formulas below is because the divider
		circuits divide by N+1

********************************************************************************
PWM Calculations
	 													 __________
Pclk/2 ===> TAT9R ===> 10 bit Counter ===>|          |
                                          | Compare  |===> Output
								    PW(M/L)xR ===>|__________|

						  Pclk/(TAPR+1)
Output Frequency = ----------------
						 (TAT9R+1) * 1024

              PW(M/L)xR * (TAT9R+1)   	 	  PW(M/L)xR
Pulse Width = --------------------- = -----------------------
				      Pclk/(TAPR+1)       Output Frequency * 1024

********************************************************************************

********************************************************************************
PPM Calculations
			    |-------------reset------------|
             |                 ___________  |
             |    TCD(H/L)R =>|          |  |
             V                | Compare  |->|
Fin ====> 16 bit Counter ====>|__________|
               |               ___________
               ==============>|          |   ________
               |              | Compare  |-> | Set  |
               | TCSx(H/L)R==>|__________|   |      |
               |               ___________   |      |==> Output
               ==============>|          |   |      |
                              | Compare  |-> | Reset|
                 TCRx(H/L)R==>|__________|   --------


					    Pclk/(TAPR+1)/(TAT1R+1)
Output Frequency = ----------------------
		 						TCD(H/L)R + 1


********************************************************************************
*/

// debug macros
#define MENU_DEBUG debug
//#define LCD_DEBUG	debug
//#define LCD_DEBUG_ASM debug

//#define FSER_DEBUG debug
//#define FSER_ISR_DEBUG	debug

//#define DAC_DEBUG	debug
#define DDS_DEBUG	debug
//#define DDS_DEBUG_PRINT	1


#use IDBLOCK_API.LIB
#use Menu_07.Lib
#use AD9834.LIB
#use MCP4822.lib

// MENU function prototypes
int	Show_Settings ( void );
int	Save_Settings ( void );
int	Read_Settings ( void );
int	Set_Def_Settings ( void );
int	Use_Settings ( void );
int	Set_Corr ( void );

int	Get_Channel ( void );
int	Channel;					// IMPORTANT: 1 relative as entered by user
   									// all array references MUST use [Channel-1]
int	Get_Frequency  ( void );
int	Get_Phase ( void );
int	Get_DutyCycle ( void );
int	Get_Time ( void );
int	Get_Count ( void );
int	Get_Volts ( void );

long	PWM_Set_Freq (  long Frequency );
void	PWM_Set_DutyCycle ( float DutyCycle );
int	PWM_Set_Width ( int Channel, int Count );
int	PWM_Set_X4 ( void );

long	PPM_Set_Freq ( long Frequency );
void	PPM_Set_DutyCycle ( float DutyCycle );
void	PPM_Set_Start ( unsigned int Count );
int 	PPM_Set_Stop ( unsigned int Count );

int	DDS_Waveform ( void );
int	DDS_Mode ( void );
int	DDSAmplitude ( float Volts );
int	DDSOffset ( float Volts );
int	DDS_Cal_Amp ( void );
int	DDS_Cal_Off ( void );

long	ARB_Set_Freq ( long Frequency );
int	ARB_GetBytes ( void );
int	ARB_Example ( void );
int	ARB_Set_State ( void );
int	ARB_Example ( void );
int	ReadARBfile ( void );

/*
To use this menu system all the functions and/or their prototypes
		must precede these declarations
Notes:
	Functions CANNOT have any parameters!
   All menu functions MUST have a return value in order to be costate capable
   	Costate capability is not necessary for this application but was
      implemented to be compatible with menu.lib.
   Entries that have "sub" levels must not have a function
*/

// these values are to tell common functions whichfeature is selected
#define DDS_INDEX 0
#define ARB_INDEX 1
#define PWM_INDEX 2
#define PPM_INDEX 3

// these values tell which parameter is being used
#define	UNITS_START_COUNT		1 			// PPM
#define	UNITS_START_DELAY		2 			// PPM
#define	UNITS_START_PHASE		3 			// PPM

#define	UNITS_STOP_COUNT		0x10		// PPM, PWM
#define	UNITS_STOP_DELAY		0x20		// PPM
#define	UNITS_STOP_PHASE		0x30		// PPM, PWM
#define	UNITS_STOP_WIDTH		0x40		// PPM, PWM
#define	UNITS_STOP_DUTYCYCLE	0x50		// PPM, PWM

#define	VOLTS_AMPLITUDE	0
#define	VOLTS_OFFSET		1

#define	ARB_BYTE_COUNT	0					// used by Get_Count
#define	ARB_CYCLES	1

const _menu MyMenu[] = {
	{	0,		"FGen Main Menu+",		NULL, 				0 },

   {	1, 		"DDS+",					NULL, 				DDS_INDEX },
   {		2,			"Channel+",			NULL, 				0 },
   {			3,			"Channel#",		Get_Channel, 		0 },
   {			3,			"Frequency",	Get_Frequency, 	0 },
   {			3,			"Phase",			Get_Phase, 			0 },
   {		2,			"Amplitude",		Get_Volts,			VOLTS_AMPLITUDE },
   {		2,			"Offset",			Get_Volts,			VOLTS_OFFSET },
	{		2,			"Waveform",			DDS_Waveform,		0 },
   {		2,			"Calibrate+",		NULL,					0 },
   {			3,			"Amplitude",	DDS_Cal_Amp,		0 },
   {			3,			"Offset",		DDS_Cal_Off,		0 },

//	{		3,			"Mode",				DDS_Mode, 			0 },

   {	1, 		"PWM+",					NULL, 				PWM_INDEX },
   {		2,			"x4 Frequency",	PWM_Set_X4,			0 },
	{		2,			"Frequency",		Get_Frequency, 	0 },
   {		2,			"Channel+",			NULL, 				0 },
   {			3,			"Channel#",		Get_Channel, 		0 },
   {			3,			"On Count",		Get_Count,	 		UNITS_STOP_COUNT },
   {			3,			"Phase Width",	Get_Phase, 			UNITS_STOP_PHASE },
   {			3,			"Pulse Width",	Get_Time,	 		UNITS_STOP_WIDTH },
   {			3,			"Duty Cycle",	Get_DutyCycle, 	UNITS_STOP_DUTYCYCLE },
//   {			3,			"Amplitude",	Get_Volts, 	VOLTS_AMPLITUDE },

   {	1, 		"PPM+",					NULL,					PPM_INDEX },
	{		2,			"Frequency",		Get_Frequency, 	0 },
   {		2,			"Channel+",			NULL, 				0 },
   {			3,			"Channel#",		Get_Channel, 		0 },
	{			3,			"Start+",		NULL,					0 },
   {				4,			"Start Count",	Get_Count, 		UNITS_START_COUNT },
   {				4,			"Start Phase",	Get_Phase, 		UNITS_START_PHASE },
	{				4,			"Start Delay",	Get_Time, 	 	UNITS_START_DELAY },
	{			3,			"Stop+",			NULL,					0 },
   {				4,			"Stop Count", Get_Count,		UNITS_STOP_COUNT },
   {				4,			"Stop Phase", Get_Phase,	 	UNITS_STOP_PHASE },
	{				4,			"Stop Delay", Get_Time,			UNITS_STOP_DELAY },
   {				4,			"Pulse Width",	Get_Time,	 	UNITS_STOP_WIDTH },
   {				4,			"Duty Cycle", Get_DutyCycle,	UNITS_STOP_DUTYCYCLE },
//   {			3,			"Amplitude",	Get_Volts, 	VOLTS_AMPLITUDE },

	{	1,			"ARB+",					NULL,					ARB_INDEX },
   {		2,			"Frequency",		Get_Frequency, 	0 },
   {			2,		"Cycles",			Get_Count,			ARB_CYCLES },
   {			2,		"Byte Count",		Get_Count,			ARB_BYTE_COUNT },
   {			2,		"Data",				ARB_GetBytes,		0 },
   {			2,		"State",				ARB_Set_State,		0 },
   {			2,		"Read ARB File",	ReadARBfile,		0 },
   {			2,		"Sample",			ARB_Example,		0 },

	{	1,			"Settings+",			NULL,					0 },
	{		2,			"Show Settings",	Show_Settings,		0 },
	{		2,			"Save Settings",	Save_Settings,		0 },
	{		2,			"Read Settings",	Read_Settings,		0 },
	{		2,			"Set Default",		Set_Def_Settings,	0 },
   {		2,			"CPU Freq. Corr",	Set_Corr,			0 },

	{  0, "", NULL, 0 }	// last entry
};

/* programmable parameters.  These will be stored in the UserBlock
 and retrieved when the program starts.  They may be used to
 preset the signals
The Start and Stop values will be one of the following:
	Count 			start and stop
	Duty Cycle		stop (%)
   Phase				start and stop (deg)
   Width				stop (usec)
*/

// IMPORTANT:
//	the VERSION must be changed whenever the structure of ChParams is modified
#define	VERSION				0xAA09
#define	UB_OFFSET			0
#define	MAX_SAVESET			8
#define	NAME_FLAG			0xA5
#define	MAX_NAME_LENGTH 	30
struct {
   int	Version;
	char	Name[MAX_NAME_LENGTH];			// NAME_FLAG, name text, NULL
	long	Frequency[4];
   int	CPU_CorrectionPPM;

	unsigned long	PWMx4Frequency;		// frequency above which the PWM output
		   // will "spread" in order to increase the frequency by a factor of 4

// These three are the Count values for the registers
	unsigned int PWM_Stop_Count[4];	// using only 0 & 1
	unsigned int PPM_Start_Count[4];
	unsigned int PPM_Stop_Count[4];

// These three are the values selected via the menu
   float PWM_Stop_Value[4];
   float PPM_Start_Value[4];
   float PPM_Stop_Value[4];

	float	Amplitude[4];
	float Offset[4];
	int	DDSwave;
	int	DDSmode;								// CW, FSK, PSK, Sweep
   unsigned	DDSPhase[2];
   float	DDS_Amp_Corr;
   float	DDS_Off_Corr;

	char	PWM_Stop_Units[4];				// using only 0 & 1
	char	PPM_Start_Units[4];
	char	PPM_Stop_Units[4];

   int	ARB_Cycles;
   int	ARB_Set;

} ChParams;

#define CHPARAMS_SIZE	256
#if CHPARAMS_SIZE < sizeof(ChParams)
	#error "ChParams structure too big"
#endif

long SaveSetNbr;

unsigned long CPU_Frequency;
float fCPU_Frequency;
float CalcPeriod;

// PPM register addresses
const int PPM_StartReg[4] = { 0x508, 0x50C, 0x518, 0x51C };
const int PPM_StopReg[4]  = { 0x50A, 0x50E, 0x51A, 0x51E };
long	PPM_DivisorValue;
float	PPM_Resolution;						// in microseconds
long	PPM_LowFrequency;						// lowest PPM frequency for TAT1R = 0

float	PWM_Resolution;						// in microseconds
// PWM spread is used when the requested frequency is too high to implement
// using "normal" PWM.  Spreading effectively multiplies the frequency by 4
// but also reduces the pulse width resolution by a factor of 4.
char PWM_X4Flag;								// PWM spread flag: must 0 or 1
#define	PWM_SPREAD_ENA		1
#define	PWMX4DIVISOR		256L			// divisor to use for lowest frequency
													//			for x4 spread

#ximport	"C:\ARBdata.txt" ARBDATA			// import ARB data file
#define	ARB_DATA_LENGTH	256			// max nbr of bytes in ARB pattern
													// byte 0 = nbr of bytes
#define	MAX_ARB_SETS		4
#define	ARB_UB_OFFSET		(UB_OFFSET + (MAX_SAVESET*CHPARAMS_SIZE))

char	ARB_Data[ARB_DATA_LENGTH];			// 1st byte is for byte count
int	ARB_Current_State;
int	ARB_ByteCount;

// prototypes for misc I/O functions
void InitIO ( void );
void TestSw ();
void WrPortI2 ( int Address, int *ShadowRegister, int DataValue );
void KnightRider ( void );

#define CPU_CORR_PPM	60

#include FGen_IO_defs-01.C

void main ()
{	int i,j;
	long lValue;

	InitIO();		// init parallel ports, DDS, DAC - MUST preceede LCDinit
	LCDinit();

   LCDprint ( " Function Generator \n" );
   LCDprint ( "      by K3PTO      \n" );
   LCDprint ( "  DDS, PWM and PPM  \n" );
	KnightRider();
   UserValue[2] = 1;							// tell ReadSettings to auto-read Set 1
	i = Read_Settings ();
	if (i) Set_Def_Settings();

// set ARB parameters
	i = ReadARBfile();
	ChParams.Frequency[ARB_INDEX] = 1000;
   ChParams.ARB_Set = 1;

// other variable initialization
   Channel = 1;

	while (1)
   {
		menu_tick ();
   }
} // main

/*******************************************************************************
********************************************************************************

	Most of the following functions use two global values to determine where
   to store the value entered by the user:
	   Channel: entered via the function Get_Channel
      	entered as 1 relative
         must use Channel-1 as array indeces
		UserValue[1]:
			DDS_INDEX
			PWM_INDEX
			PPM_INDEX
         ARB_INDEX
using Get_Frequency as an example, it gets the initial value of the frequency
from ChParams.Frequency[ UserValue[1] ]
*/



// ==================================================================
// ===================== GUI specific functions =====================
// ==================================================================

int Get_Channel ( void )
{	long templong;

   templong = Channel;
				// 12345
	LCDprint ( "  =   " );
	GetLong_B ( &templong, 2, 4 );
   Channel = (int)templong;
   if ( Channel == 1  &&   UserValue[1] == PPM_INDEX)
   {
	// 1: PPM-0
		WrPortI ( PCALR, &PCALRShadow, PCALRShadow &~0x03 ); // PC0 = out 0
		WrPortI ( PCALR, &PCALRShadow, PCALRShadow |0x02 ); // PC0=alt out 2=PPM0
		BitWrPortI ( PCFR, &PCFRShadow, 1, 0 ); // enable alt function on PC0
		BitWrPortI ( PCDDR, &PCDDRShadow, 1, 0 ); // enable PC0 as output
   }
   return 1;
} // Get_Channel


void ShowLCDstring ( void )
{  int ColumnSave, LineSave;

	ColumnSave = LCDcolNbr;					// save cursor
   LineSave = LCDlineNbr;					//			position
	LCDcursorPos ( LCDlineNbr+1, 1 );
   LCDprint ( LCDstring );
	LCDcursorPos ( LineSave, ColumnSave );
} // ShowLCDstring


int Get_Frequency ( void )
{
	long CalcFreq, Frequency, PrevFrequency;
   float Period;
   int UV1;


   UV1 = UserValue[1];
	Frequency = ChParams.Frequency[UV1]; // get current freq
				// 12345678901234567890
            //       +12345678Hz
	LCDprint ( "Freq=          Hz"  );
   PrevFrequency = 0;						// force mismatch for 1st time
	while ( !GetLong(&Frequency,9,7) )	// get new value
	{
		if ( Frequency <= 0 ) Frequency = 1000; // do not allow 0!
// NOTE: even though Set_(PWM/PPM)_Freq returns the actual frequency, the value
// should not be used here because the operation of the GetLong function will
// never allow the frequency to change.
      if ( Frequency != PrevFrequency )
		{	PrevFrequency = Frequency;		// setup for next time

			switch (UV1)
         {
			case PWM_INDEX:
         	CalcFreq = PWM_Set_Freq(Frequency);
				Period = 1e6/(float)CalcFreq;
   			if (PWM_X4Flag)
   				sprintf ( LCDstring, "Act =  %08ld  x4\n\rPer = %8.4fus",
               			CalcFreq, Period );
            else
   				sprintf ( LCDstring, "Act =  %08ld    \n\rPer = %8.4fus    ",
               			CalcFreq, Period );
                        // extra spaces following %08ld are needed to blank x4
				ShowLCDstring ();
				break;

			case PPM_INDEX:
				// enable PC0 as alternate output 2 = PPM
				WrPortI ( PCALR, &PCALRShadow, (PCALRShadow & ~0x03)|0x02 );
				BitWrPortI ( PCFR, &PCFRShadow, 1, 0 ); // enable alt function on PC0
				BitWrPortI ( PCDDR, &PCDDRShadow, 1, 0 ); // enable PC0 as output

         	CalcFreq = PPM_Set_Freq(Frequency);
				Period = 1e6/(float)CalcFreq;
  				sprintf ( LCDstring, "Act =  %08ld    \n\rPer = %8.4fus",
               			CalcFreq, Period );
				ShowLCDstring ();
            break;

			case DDS_INDEX:
			   DDSsetFreq ( Channel-1, Frequency, ChParams.DDSPhase[Channel-1] );
            									// set DDS frequency
            ChParams.Frequency[DDS_INDEX] = Frequency; // save it
            break;

			case ARB_INDEX:
				CalcFreq = ARB_Set_Freq(Frequency);
				Period = 1e6/(float)CalcFreq;
  				sprintf ( LCDstring, "Act =  %08ld    \n\rPer = %8.4fus",
               			CalcFreq, Period );
				ShowLCDstring ();
         	break;
         } // switch (UserValue[1])
		} // Frequency != PrevFrequency
   } // while (!GetLong)
   return 1;
} // Get_Frequency


int Get_DutyCycle ( void )
{	float DutyCycle, PrevDutyCycle;
   int i, UV1;

   UV1 = UserValue[1];
//  calculate current Duty Cycle
	if ( UV1 == PWM_INDEX )
   {
     	DutyCycle = (float)ChParams.PWM_Stop_Count[Channel-1]/10.24;
		ChParams.PWM_Stop_Units[Channel-1] = UNITS_STOP_DUTYCYCLE;
	} // PWM_INDEX

   else //  must be PPM
   {
      DutyCycle = (float)(ChParams.PPM_Stop_Count[Channel-1]-ChParams.PPM_Start_Count[Channel-1]);
		DutyCycle /= (float)PPM_DivisorValue;
		DutyCycle *= 100.0;
   	ChParams.PPM_Stop_Units[Channel-1] = UNITS_STOP_DUTYCYCLE;
   } // PPM_INDEX

				// 12345678901234567890
            //            +23.56
	LCDprint ( "DutyCycle =      %"  );
   PrevDutyCycle = 0.0;						// force mismatch for 1st time
	while ( !GetFloat ( &DutyCycle, 6, 2, 12 ) )
   {
		if ( DutyCycle != PrevDutyCycle )
      {  if ( DutyCycle <= 0.0  ||  DutyCycle >=100.0 ) DutyCycle = 50.0;
			PrevDutyCycle = DutyCycle;		// save for next time

 			if (UV1 == PWM_INDEX)
         {	PWM_Set_DutyCycle ( DutyCycle );
		      ChParams.PWM_Stop_Value[Channel-1] = DutyCycle;
         }
			if (UV1 == PPM_INDEX)
         {	PPM_Set_DutyCycle ( DutyCycle );
		      ChParams.PPM_Stop_Value[Channel-1] = DutyCycle;
         }
      } // DutyCycle != PrevDutyCycle
   } //  while (!GetFloat)
	return 1;
} // Get_DutyCycle


int Get_Time ( void )
{	unsigned int Count, i, UV1;
   float tempfloat, Resolution;
   float PrevTime, Time, CalcTime;

   UV1 = UserValue[1];
// get current on time value
	if ( UV1 == PWM_INDEX )
	{	Resolution = PWM_Resolution;
		Time = Resolution*(float)ChParams.PWM_Stop_Count[Channel-1];
		ChParams.PWM_Stop_Units[Channel-1] = UNITS_STOP_WIDTH;
		LCDprint ( "Width =       \344s"  );		// 344 = micro symbol

   } // PWM_INDEX
   else // PPM_INDEX
   {
      Resolution = PPM_Resolution;
		switch (UserValue[MenuLevel])
      {
		case UNITS_START_DELAY:
      	Time = Resolution*(float)ChParams.PPM_Start_Count[Channel-1];
			ChParams.PPM_Start_Units[Channel-1] = UNITS_START_DELAY;
			LCDprint ( "Delay =       \344s"  );		// 344 = micro symbol
      	break;
		case UNITS_STOP_DELAY:
      	Time = Resolution*(float)ChParams.PPM_Stop_Count[Channel-1];
			ChParams.PPM_Stop_Units[Channel-1] = UNITS_STOP_DELAY;
			LCDprint ( "Delay =       \344s"  );		// 344 = micro symbol
      	break;
		case UNITS_STOP_WIDTH:
      	Time = Resolution*(float)ChParams.PPM_Stop_Count[Channel-1];
			ChParams.PPM_Stop_Units[Channel-1] = UNITS_STOP_WIDTH;
			LCDprint ( "Width =       \344s"  );		// 344 = micro symbol
      	break;
      }
   } // PPM_INDEX
//   Time += 0.5;								// round value up

				// 12345678901234567890
            //         +2345.7us
//	LCDprint ( "Width  =       \344s"  );
   PrevTime = 0.0;							// force mismatch for 1st time
	while ( !GetFloat ( &Time, 7, 1, 9 ) )
	{
		if ( PrevTime != Time )
      {  PrevTime = Time;					// setup for next time
			if ( Time < 0.0 ) Time = 0.0; // do not allow negative
			tempfloat = Time/Resolution;
			Count = (int)(tempfloat+0.5);


			if (UV1 == PWM_INDEX)
         {	Count = PWM_Set_Width ( Channel, Count );
		      ChParams.PWM_Stop_Value[Channel-1] = Time;
         }
			else	// PPM_INDEX
         {
				switch (UserValue[MenuLevel])
		      {
				case UNITS_START_DELAY:
               PPM_Set_Start ( Count );
			      ChParams.PPM_Start_Value[Channel-1] = Time;
               // no change in stop value if using UNITS_STOP_PHASE
		      	break;
				case UNITS_STOP_DELAY:
				case UNITS_STOP_WIDTH:
               PPM_Set_Stop ( Count );
			      ChParams.PPM_Stop_Value[Channel-1] = Time;
		      	break;
		      }
         } // PPM_INDEX

			CalcTime = Resolution * Count;
									 // 12345678901234567890
			sprintf ( LCDstring, "  Act =  %6.1fus", CalcTime );
			ShowLCDstring ();

      } // PrevTime != Time
   } // while (!GetLong)
   return 1;
} // Get_Time


int Get_Count ( void )
{	int i, UV1;
   long PrevCount, Count;

   UV1 = UserValue[1];
// get current Count value
	switch ( UV1 )
   {
	case PWM_INDEX:
     	Count = ChParams.PWM_Stop_Count[Channel-1];
		ChParams.PWM_Stop_Units[Channel-1] = UNITS_STOP_COUNT;
		break;
   case PPM_INDEX:
		if (UserValue[MenuLevel] == UNITS_START_COUNT )
      { 	Count = ChParams.PPM_Start_Count[Channel-1];
   		ChParams.PPM_Start_Units[Channel-1] = UNITS_START_COUNT;
      }
		else
      { 	Count = ChParams.PPM_Stop_Count[Channel-1];
   		ChParams.PPM_Stop_Units[Channel-1] = UNITS_STOP_COUNT;
      }
		break;
   case ARB_INDEX:
		if (UserValue[MenuLevel] == ARB_BYTE_COUNT )
	      Count = ARB_ByteCount;
      else
	      Count = ChParams.ARB_Cycles;
   	break;
   }

				// 12345678901234567890
            //    +123456
	LCDprint ( "  =       "  );
   PrevCount = 0;								// force mismatch for 1st time
	while ( !( GetLong(&Count,7,4) ) ) 	// get new value
	{
		if ( Count != PrevCount )
      {
			switch ( UV1 )
   		{
			case PWM_INDEX:
			if ( Count < 0 ) Count = 0; 	// do not allow negative!
           	Count = PWM_Set_Width ( Channel, (int)Count );
		      ChParams.PWM_Stop_Count[Channel-1] = (int)Count;
				break;
			case PPM_INDEX:
			if ( Count < 0 ) Count = 0; 	// do not allow negative!
				if (UserValue[MenuLevel] == UNITS_START_COUNT )
      		{ 	PPM_Set_Start ( (int)Count );
			      ChParams.PPM_Start_Units[Channel-1] = (int)Count;
            }
            else								// must be UNITS_STOP_COUNT
	         {	PPM_Set_Stop ( (int)Count );
			      ChParams.PPM_Stop_Units[Channel-1] = (int)Count;
            }
            break;
			case ARB_INDEX:
				if (UserValue[MenuLevel] == ARB_BYTE_COUNT )
            {  if ( Count < 0 ) Count = 0; 	// do not allow negative!
					if ( Count >= ARB_DATA_LENGTH ) Count = ARB_DATA_LENGTH-1;
	   		   ARB_ByteCount = (int)Count;
            }
		      else
	   		{   ChParams.ARB_Cycles = (int)Count;
            }
            break;
   		}
			PrevCount = Count;		  		// save for next time
      } // Count != PrevCount
   } // while (!GetLong)
   return 1;
} // Get_Count


int Get_Phase ( void )
{	float Phase, PrevPhase;
   float tempfloat;
   unsigned int Count, Width, UV1;
   long OnCount, StopCount, StartCount;

   UV1 = UserValue[1];
	if ( UV1 == PWM_INDEX )
   {
     	Phase = (float)ChParams.PWM_Stop_Count[Channel-1];
      Phase /= 1024.0;
      Phase *= 360.0;
		ChParams.PWM_Stop_Units[Channel-1] = UNITS_STOP_PHASE;
	} // PWM_INDEX

	if ( UV1 == PPM_INDEX )
   {
// PPM_DivisorValue
		Width = ChParams.PPM_Stop_Count[Channel-1] - ChParams.PPM_Start_Count[Channel-1];
		switch (UserValue[MenuLevel])
      {
		case UNITS_START_PHASE:
	     	Phase = (float)ChParams.PPM_Start_Count[Channel-1];
	   	ChParams.PPM_Start_Units[Channel-1] = UNITS_START_PHASE;
      	break;
      case UNITS_STOP_PHASE:
	     	Phase = (float)ChParams.PPM_Stop_Count[Channel-1];
	   	ChParams.PPM_Stop_Units[Channel-1] = UNITS_STOP_PHASE;
      	break;
      }
      Phase /= (float)PPM_DivisorValue;
      Phase *= 360.0;
   } // PPM_INDEX

	if ( UV1 == DDS_INDEX )
	{
		Phase = ChParams.DDSPhase[Channel-1];
   } // DDS_INDEX

				// 12345678901234567890
            //        +123.45
	LCDprint ( "Angle =       \337" );	// 337 = degrees symbol
	PrevPhase = -1;							// force mismatch for 1st time
	while ( !GetFloat ( &Phase, 7, 2, 8 ) )
	{
		if ( Phase != PrevPhase )
      {  if ( Phase < 0.0 ) Phase = 0.0;
        	if ( Phase >= 360.0 ) Phase = 359.9;
			PrevPhase = Phase;				// save for next time

			if ( UV1 == PWM_INDEX )
   	   {
				tempfloat = ( Phase/360.0 ) * 1024.0;
            Count = (int)(tempfloat+0.5);
				Count = PWM_Set_Width ( Channel, Count );
		      ChParams.PWM_Stop_Value[Channel-1] = Phase;
		      tempfloat = (float)Count/1024.0;
	     		tempfloat *= 360.0;
											// 12345678901234567890
				sprintf ( LCDstring, "\r Act = %7.2f\337", tempfloat );
  	      	ShowLCDstring();
			}

			if ( UV1 == PPM_INDEX )
			{
        		tempfloat = Phase/360.0;
				tempfloat *= (float)PPM_DivisorValue;
            Count = (int)(tempfloat+0.5);
            if ( Count >= PPM_DivisorValue ) Count = Count-(unsigned int)PPM_DivisorValue;

				switch (UserValue[MenuLevel])
   		   {
				case UNITS_START_PHASE:
            	PPM_Set_Start ( Count );
			      ChParams.PPM_Start_Value[Channel-1] = Phase;
      			break;

		      case UNITS_STOP_PHASE:
            	PPM_Set_Stop ( Count );
			      ChParams.PPM_Stop_Value[Channel-1] = Phase;
   	   		break;
	   	   } // switch (UserValue[MenuLevel])
		      tempfloat = (float)Count/(float)PPM_DivisorValue;
	     		tempfloat *= 360.0;
											// 12345678901234567890
				sprintf ( LCDstring, "\r Act = %7.2f\337", tempfloat );
  	      	ShowLCDstring();
			} // PPM_INDEX

			if ( UV1 == DDS_INDEX )
			{
				ChParams.DDSPhase[Channel-1] = (int)Phase;
			   DDSsetFreq ( Channel-1, ChParams.Frequency[DDS_INDEX], ChParams.DDSPhase[Channel-1] );
		   } // DDS_INDEX


      } // Phase != PrevPhase
   } // !GetFloat
   return 1;
} // Get_Phase


int Get_Volts ( void )
{	float Volts, PrevVolts;
	int i,UV1;

	UV1 = UserValue[1];						// so we don't have to use array references
// get current value and print subtitle
 	if ( UserValue[MenuLevel] == VOLTS_AMPLITUDE )
	{	Volts = ChParams.Amplitude[UV1];
					// 12345678901234567890
            	//             +1.23
		LCDprint ( "Amplitude =      "  );
   }
   else
	{	Volts = ChParams.Offset[UV1];
		LCDprint ( "   Offset =      "  );
   }

// get new value
	PrevVolts = 1e6;							// force mismatch for 1st time
	while ( !GetFloat ( &Volts, 5, 2, 13 ) )
   {
		if ( Volts != PrevVolts )
      {	PrevVolts = Volts;				// save for next time
			if (UV1 == DDS_INDEX)
         {
				if ( UserValue[MenuLevel] == VOLTS_AMPLITUDE )
				{	DDSAmplitude ( Volts );
				   ChParams.Amplitude[UV1] = Volts;
            }
				else
				{	DDSOffset ( Volts );
				   ChParams.Offset[UV1] = Volts;
            }
         } // UserValue[1] == DDS_INDEX
      } // Amplitude != PrevAmplitude
   } // Volts != PrevVolts

   return 1;
} // Get_Volts



// ==================================================================
// ===================== PWM specific functions =====================
// ==================================================================

long PWM_Set_Freq ( long Frequency )
{	long Divisor, calcFreq, PWMdivisor;
   int i, iChannel;
   float tempfloat;
   unsigned int Count;

   PWMdivisor = CPU_Frequency/(TAPRShadow+1);	// frequency into TA9

	if ( Frequency < ChParams.PWMx4Frequency )
   {	PWM_X4Flag = 0;
	   PWMdivisor /= 1024;
   }
   else
   {	PWM_X4Flag = PWM_SPREAD_ENA;
	   PWMdivisor /= 256;
   }

  	Divisor = PWMdivisor/Frequency; 		// initial divisor
   if ( Divisor > 256 ) Divisor = 256; // maximum
   if ( Divisor < 1 ) Divisor = 1;
   calcFreq = PWMdivisor/Divisor; 		// calculate actual frequency
   PWM_Resolution = (float)Divisor/(fCPU_Frequency/(TAPRShadow+1));
   PWM_Resolution *= 1e6;					// make usec

   if ( PWM_X4Flag ) PWM_Resolution /= 4.0;

// now set the width for the new frequency
// IMPORTANT - the hardware uses PWM channels 1 & 2
	for ( iChannel=1; iChannel<=2; iChannel++ ) // for both channels
   {
     	Count = ChParams.PWM_Stop_Count[iChannel-1];
		switch (ChParams.PWM_Stop_Units[iChannel-1])
      {
		case UNITS_STOP_COUNT:
		case UNITS_STOP_PHASE:
		case UNITS_STOP_DUTYCYCLE:
			break;
		case UNITS_STOP_WIDTH:
			tempfloat = ChParams.PWM_Stop_Value[iChannel-1]/PWM_Resolution; // calculate count
			Count = (int)(tempfloat+0.5); // adjust for rounding
			break;
      }
		PWM_Set_Width ( iChannel, Count );
   } // for both channels

   WrPortI ( TAT9R, &TAT9RShadow, (int)Divisor-1 ); // set frequency
	ChParams.Frequency[PWM_INDEX] = (long)calcFreq; // save new frequency

   return  (calcFreq);
} // PWM_Set_Freq


void PWM_Set_DutyCycle ( float DutyCycle )
{	int Count;

	DutyCycle *= 10.24;						// (DytyCycle/100)*1024 = DutyCycle/10.24
	Count = (int)(DutyCycle+0.5);			// make int
	PWM_Set_Width ( Channel, Count );
// show actual Duty Cycle
	if ( PWM_X4Flag )
   {	Count /= 4;								// adjust for removing 2 LSBs
   	DutyCycle = Count*100.0/256.0;
   }
   else
   {	DutyCycle = Count/10.24;
   }
	sprintf ( LCDstring, "\r      Act = %05.2f%%", DutyCycle );
	ShowLCDstring();

} // PWM_Set_DutyCycle


int PWM_Set_Width ( int iChannel, int Count )
{	int i;

	if ( Count <= 0 ) Count = 1;
	if ( Count > 1024 ) Count = 1024;
   ChParams.PWM_Stop_Count[iChannel-1] = Count; // save it
   Count -= 1;									// pulse is N+1 counts
   Count <<= 6;								// move into bits 15..6;
   Count &= 0xFFC0;							// keep highest 10 bits
   Count |= PWM_X4Flag;						// insert spread bit
	i = (iChannel-1)*2;						// offset to PWLxR
   WrPortI2 ( PWL1R+i, NULL, Count );
   return ChParams.PWM_Stop_Count[iChannel-1];
} // PWM_Set_Width


int PWM_Set_X4 ( void )
{
	long Frequency, calcFreq, PrevFrequency, LowLimit, HighLimit;
	unsigned long PWMdivisor, Divisor;

	LowLimit = CPU_Frequency / (256L*256*(TAPRShadow+1));
   											// lowest frequency with spreading on
	HighLimit = CPU_Frequency / (1024*(TAPRShadow+1));
   											// highest frequency with spreading off
	sprintf ( LCDstring, "%3ld < Freq < %6ld\n\r", LowLimit, HighLimit );
	LCDprint ( LCDstring );

	Frequency = ChParams.PWMx4Frequency; // get current freq
				// 12345678901234567890
            //       +12345678Hz
   LCDprint ( "Freq=          Hz"  );
   PrevFrequency = 0;						// force mismatch for 1st time
	while ( !GetLong(&Frequency,9,7) )	// get new value
	{
		if ( Frequency < LowLimit ) Frequency = LowLimit;
		if ( Frequency > HighLimit ) Frequency = HighLimit;
      if ( Frequency != PrevFrequency )
		{	PrevFrequency = Frequency;		// setup for next time
		}
   } // while (!GetLong)
   ChParams.PWMx4Frequency = Frequency;
   return (1);
} // PWM_Set_X4



// ==================================================================
// ===================== PPM specific functions =====================
// ==================================================================


long PPM_Set_Freq ( long Frequency )
{	long calcFreq;
	int iChannel, TAT1Rvalue;
   float tempfloat;
   long StartCount, StopCount;

	if ( Frequency >= PPM_LowFrequency )
   {	TAT1RShadow = 0;
   }
   else
   {
	   PPM_DivisorValue = CPU_Frequency/(TAPRShadow+1); // freq into PPM UpCounter when TAT1R = /1
	   PPM_DivisorValue /= Frequency;	// divisor to get requested frequency
      TAT1RShadow = TAT1Rvalue = (int)(PPM_DivisorValue>>16); //  calculate TAT1R value
      if ( TAT1Rvalue > 255 ) TAT1RShadow = 255;
   }
	tempfloat = (TAPRShadow+1)*(TAT1RShadow+1); // total divisor driving Timer C
   tempfloat = fCPU_Frequency/tempfloat; // frequency driving Timer C
   PPM_Resolution = 1e6/tempfloat;		// time resolution in usec
			// used for pulse width, start delay and stop delay

   PPM_DivisorValue = (long)tempfloat/Frequency; // divisor to get requested frequency
	if ( PPM_DivisorValue > 65536 ) PPM_DivisorValue = 65536;
   if ( PPM_DivisorValue <= 2 ) PPM_DivisorValue = 2;

	for ( iChannel=1; iChannel<=4; iChannel++ ) // for all channels
   {
     	StartCount = ChParams.PPM_Start_Count[iChannel-1]; // current values
	// calculate the start count for the new frequency
		switch (ChParams.PPM_Start_Units[iChannel-1])
      {
		case UNITS_START_COUNT:
      	break;
		case UNITS_START_DELAY:
			StartCount = (long)(ChParams.PPM_Start_Value[iChannel-1]/PPM_Resolution);
			break;
		case UNITS_START_PHASE:
			StartCount = (long)(ChParams.PPM_Start_Value[iChannel-1]*PPM_DivisorValue/360);
			break;
      }

     	StopCount = ChParams.PPM_Stop_Count[iChannel-1];
	// calculate the stop count for the new frequency
		switch (ChParams.PPM_Stop_Units[iChannel-1])
      {
		case UNITS_STOP_COUNT:
      	break;
		case UNITS_STOP_DELAY:
			StopCount = (long)(ChParams.PPM_Stop_Value[iChannel-1]/PPM_Resolution);
      	break;
		case UNITS_STOP_PHASE:
			StopCount = (long)(ChParams.PPM_Stop_Value[iChannel-1]*PPM_DivisorValue/360);
			break;
		case UNITS_STOP_WIDTH:
			StopCount = (long)(ChParams.PPM_Stop_Value[iChannel-1]/PPM_Resolution);
         StopCount += StartCount;
			break;
		case UNITS_STOP_DUTYCYCLE:
			StopCount = (long)(ChParams.PPM_Stop_Value[iChannel-1]*PPM_DivisorValue/100.0);
         StopCount += StartCount;
			break;
      }

		WrPortI2 ( PPM_StartReg[iChannel-1], NULL, (int)StartCount );
     	ChParams.PPM_Start_Count[iChannel-1] = (int)StartCount;
		WrPortI2 ( PPM_StopReg[iChannel-1], NULL, (int)StopCount );
     	ChParams.PPM_Stop_Count[iChannel-1] = (int)StopCount;
   } // for all channels

   WrPortI ( TAT1R, NULL, TAT1RShadow );
   WrPortI2 ( TCDLR, NULL, (int)(PPM_DivisorValue-1) ); // set frequency
   calcFreq = CPU_Frequency/PPM_DivisorValue/(TAPRShadow+1)/(TAT1RShadow+1);
   												// calculate actual frequency
	ChParams.Frequency[PPM_INDEX] = calcFreq; //  save new frequency

   return  (calcFreq);
} // PPM_Set_Freq


void PPM_Set_DutyCycle ( float DutyCycle )
{  unsigned int Count;
	float tempfloat;

   DutyCycle /= 100.0;
   DutyCycle *= (float)PPM_DivisorValue;
	Count = (int)(DutyCycle+0.5);			// make int
   Count += ChParams.PPM_Start_Count[Channel-1]; // offset from start
   if ( Count >= PPM_DivisorValue ) Count = Count - (unsigned int)PPM_DivisorValue;
	WrPortI2 ( PPM_StopReg[Channel-1], NULL, Count );
   ChParams.PPM_Stop_Count[Channel-1] = Count; // save it
// calculate and display actual duty cycle
	Count -= ChParams.PPM_Start_Count[Channel-1]; // pulse width
	tempfloat = (float)Count/(float)PPM_DivisorValue;
   tempfloat *= 100.0;
	sprintf ( LCDstring, "\r      Act = %05.2f%%", tempfloat );
	ShowLCDstring();

} // PPM_Set_DutyCycle


/* valid PPM start and stop modes:
UNITS_START_COUNT		UNITS_START_PHASE		UNITS_START_DELAY

UNITS_STOP_COUNT		UNITS_STOP_PHASE		UNITS_STOP_DELAY
UNITS_STOP_WIDTH		UNITS_STOP_DUTYCYCLE
*/

void PPM_Set_Start ( unsigned int Count )
{	int i, Width;

	Width = ChParams.PPM_Stop_Count[Channel-1] - ChParams.PPM_Start_Count[Channel-1];
						// Width needed for UNITS_STOP_DUTYCYCLE and UNITS_STOP_WIDTH
	if ( Width < 0 ) Width += (unsigned int)PPM_DivisorValue;
   if ( Count >= PPM_DivisorValue ) Count -= (unsigned int)PPM_DivisorValue;
	WrPortI2 ( PPM_StartReg[Channel-1], NULL, Count );
  	ChParams.PPM_Start_Count[Channel-1] = Count;

	i = ChParams.PPM_Stop_Units[Channel-1];
   if ( i==UNITS_STOP_DUTYCYCLE  ||  i==UNITS_STOP_WIDTH )
	{	// need to move stop value to keep same duty cycle or width
      Count += Width;						// new stop value
      if ( Count >= PPM_DivisorValue ) Count -= (unsigned int)PPM_DivisorValue;
		WrPortI2 ( PPM_StopReg[Channel-1], NULL, Count );
	  	ChParams.PPM_Stop_Count[Channel-1] = Count;
   }
} // PPM_Set_Start


int PPM_Set_Stop ( unsigned int Count )
{	int i;

	i = ChParams.PPM_Stop_Units[Channel-1];
	if ( i==UNITS_STOP_WIDTH  ||  i==UNITS_STOP_DUTYCYCLE)
   {
		Count += ChParams.PPM_Start_Count[Channel-1]; // adjust for start position
   }
	if ( Count >= PPM_DivisorValue ) Count -= (unsigned int)PPM_DivisorValue;
	WrPortI2 ( PPM_StopReg[Channel-1], NULL, Count );
	ChParams.PPM_Stop_Count[Channel-1] = Count;
   return Count;
} // PPM_Set_Stop



// ==================================================================
// ===================== DDS specific functions =====================
// ==================================================================

const char WaveFormList[][LIST_ITEM_SIZE] = { "Sine", "Triangle", "(Square)", "(Square/2)" };
int DDS_Waveform ( void )
{
	ListItemState = ChParams.DDSwave;
   if ( ListItemState < 0  ||  ListItemState > 3 ) ListItemState = 0;
	while ( !GetListEntry_B ( (const char*)WaveFormList, 4, SINGLE_ITEM ) );
	ChParams.DDSwave = ListItemState;
   switch (ListItemState)
   {
   	case 1:  DDSsetWaveForm ( OUT_SINE ); break;
	   case 2:  DDSsetWaveForm ( OUT_TRIANGLE ); break;
	   case 4:  DDSsetWaveForm ( OUT_SQUARE ); break;
	   case 8:  DDSsetWaveForm ( OUT_SQUARE_DIV2 ); break;
   }
	return 1;
}

const char ModeList[][LIST_ITEM_SIZE] = { "CW-F0", "CW-F1", "FSK/PSK", "Ramp1", "Ramp2" };
int DDS_Mode ( void )
{	int i;

	ListItemState = ChParams.DDSmode;
	while ( !GetListEntry_B ( (const char *)ModeList, 5, SINGLE_ITEM ) );
   ChParams.DDSmode = ListItemState;
   return 1;
}

int DDSAmplitude ( float Volts )
{	int i;

	Volts = Volts / ChParams.DDS_Amp_Corr;

	if ( Volts < 0.0 )
   {	Volts = 0.0;
   	LED (0,1);
   }
	else if ( Volts > 1.2 )
   {	Volts = 1.2;
   	LED (0,1);
   }
   else
      LED (0,0);

#if DDS_DEBUG_PRINT & 2
	printf ( "\n\rSetAmplitude: %f\n\r", Volts );
#endif

	Volts = 1.2-Volts;
	i = DAC_volts ( DDS_AMP_DAC, Volts );
   return 0;
} // DDSAmplitude

#define DDS_MAX_OFFSET	1.9
int DDSOffset ( float Volts )
{	int i;

	Volts = Volts / ChParams.DDS_Off_Corr;

	if ( Volts < -DDS_MAX_OFFSET )
   {	Volts = -DDS_MAX_OFFSET;
   	LED (0,1);
   }
	else if ( Volts > DDS_MAX_OFFSET )
   {	Volts = DDS_MAX_OFFSET;
   	LED (0,1);
   }
   else
      LED (0,0);

#if DDS_DEBUG_PRINT & 2
	printf ( "\n\rSetOffset: %f\n\r", Volts );
#endif

	Volts = (DDS_MAX_OFFSET - Volts)/DDS_MAX_OFFSET;
	i = DAC_volts ( DDS_OFF_DAC, Volts );
   return i;
} // DDSOffset

#define	CAL_DDS_AMPLITUDE	1.0
int	DDS_Cal_Amp ( void )
{	float	Amplitude;

	Amplitude = CAL_DDS_AMPLITUDE;
   DDSAmplitude ( CAL_DDS_AMPLITUDE ); // uncalibrated
	LCDprint ( "Enter P to P Amp\n" );
	while ( !GetFloat(&Amplitude,5,2, 1) ); // get CAL_DDS_AMPLITUDE reading
   ChParams.DDS_Amp_Corr *= Amplitude;
   DDSAmplitude ( CAL_DDS_AMPLITUDE ); // calibrated
	return 1;
} // DDS_Cal_Amp

#define	CAL_DDS_OFFSET	1.0
int	DDS_Cal_Off ( void )
{	float	Offset;

   DDSAmplitude ( 0.0 );
   Offset = CAL_DDS_OFFSET;
	DDSOffset ( CAL_DDS_OFFSET );			// uncalibrated
	LCDprint ( "Enter Offset\n" );
	while ( !GetFloat(&Offset,5,2, 1) ); // get CAL_DDS_OFFSET reading
   ChParams.DDS_Off_Corr *= Offset;
   DDSOffset ( CAL_DDS_OFFSET );	  		// calibrated
	return 1;
} // DDS_Cal_Off

// ==================================================================
// ===================== ARB specific functions =====================
// ==================================================================


long ARB_Set_Freq ( long Frequency )
{	long ActualFrequency;

	ActualFrequency = fSerOpen ( FSER_D, Frequency, 2, FSER_RXC );
   ChParams.Frequency[ARB_INDEX] = ActualFrequency;
   return ActualFrequency;
} // ARB_Set_Freq

// uses ARB_Data and ARB_DATA_LENGTH, byte 0 = nbr of data bytes in pattern
int  ARB_GetBytes ( void )
{	char *Pointer;

   ARB_Data[0] = ARB_ByteCount;			// insert byte count into pattern
	Pointer = ARB_Data;						// point to start of array

	LCDline (1);
	sprintf ( LCDstring, "Data: %3d bytes ", ARB_Data[0] );
	LCDprint ( LCDstring );

   while ( !GetHexBytes(Pointer) );
   return 1;

} // ARB_GetBytes

const char Arb_States[2][LIST_ITEM_SIZE] = { "Run ", "Stop" };
int  ARB_Set_State ( void )
{	int i;

	i = fSerTxActive ( FSER_D );
   if (i==1) ListItemState = 1;
   else ListItemState = 2;

	BitWrPortI ( PCDR, &PCDRShadow, 0, 0 ); // PC0 = 0
	while ( !GetListEntry ( (const char*)Arb_States, 2, SINGLE_ITEM ) );
	if ( ListItemState & 1 )				// if run
   {
		WrPortI ( PCALR, &PCALRShadow, PCALRShadow &~0x03 ); // PC0 = alt out 0
		BitWrPortI ( PCFR, &PCFRShadow, 1, 0 ); // enable alt function on PC0
		BitWrPortI ( PCDDR, &PCDDRShadow, 1, 0 ); // enable PC0 as output

   	fSerTxCyclesFG[FSER_D] = ChParams.ARB_Cycles; // set up nbr of cycles
		ARB_Set_Freq ( ChParams.Frequency[ARB_INDEX] ); // and bit rate
	   i = fSerWrite ( FSER_D, (far char*)paddr(&ARB_Data[1]), (int)ARB_Data[0] );
  		LCDprint ( "Running" );
   }
   else
   {
//		BitWrPortI ( PCFR, &PCFRShadow, 0, 0 ); // disable alt function on PC0
		fSerTxCyclesFG[FSER_D] = 1;		// stop the ARB
      fSerStop (FSER_D);
  		LCDprint ( "Stopped" );
   }
	MsDelay (2000);
   return 1;
} // ARB_Set_State


const char ARBsample[] = { 0xAA, 0xF0, 0xAA, 0xF0 };
int ARB_Example ( void )
{	int i,j;

	memcpy ( &ARB_Data[1], ARBsample, sizeof(ARBsample) );
	ARB_ByteCount = ARB_Data[0] = sizeof(ARBsample);
	LCDprint ( "Use State to start\nUse Data to Edit" );
	MsDelay (2000);
   return 1;
} // ARB_Example


// the following two functions return the digit value if successful
// and a -1 if not successful

int IsDigit ( int Digit )
{	if ( Digit >= '0'  &&  Digit <= '9' ) return Digit - '0';
	return -1;
}

int IsHex ( int Digit )
{	if ( Digit >= '0'  &&  Digit <= '9' ) return Digit - '0';
	if ( Digit >= 'A'  &&  Digit <= 'F' ) return Digit - 'A' + 10;
	if ( Digit >= 'a'  &&  Digit <= 'f' ) return Digit - 'a' + 10;
	return -1;
}

/* The pointer to the ASCII file is ARBDATA
 the text file format is:
	Line 1:
	 	byte 0:	A
   	bytes 1-3: byte count of data
	Lines 2-n: data bytes in two digit Hex ASCII format
   	single space between bytes
      multiple lines may be used
example:
A6
AA F0 AA F0 00 00

*/

int ReadARBfile ( void )
{	int	retvalue, i;
	char	*pARB_Data;
	char far	*pText;
	char far	*pTextEnd;
   long	ByteCount;

	LCDclear();
   LCDprint ( "Read ARB data file" );
	xmem2root ( &ByteCount, ARBDATA, 4 );
	retvalue = 0;								// default to failure
   memset ( ARB_Data, 0, ARB_DATA_LENGTH ); // clear data area
	pText = (char far *)ARBDATA+4;
   pTextEnd = pText + ByteCount;			// address of end of text file

// get byte count
   if ( *pText++ != 'A' )
   {	LCDprint ( "\nFail!" );
   	MsDelay ( 2000 );
   	return 0;
   }

	ARB_ByteCount = 0;						// init byte count
   while ( (i=IsDigit(*pText++)) >= 0 )
   {	ARB_ByteCount *= 10;
   	ARB_ByteCount += i;
   }
	ARB_Data[0] = ARB_ByteCount;

// get the data values
   pARB_Data = &ARB_Data[1];				// point to area to store data bytes

	while ( ARB_ByteCount--  &&  pText <= pTextEnd )
   {
		while ( (i=IsHex(*pText++)) == -1 ); // find valid hex character
		*pARB_Data = i*16;					// MS digit
		*pARB_Data += IsHex(*pText++);	// LS digit
		pARB_Data++;							// point to location for next byte
   }
	ARB_ByteCount = ARB_Data[0];			// retrieve byte count
	LCDprint ( "\nSuccess" );
  	MsDelay ( 2000 );
   return 1;
} // ReadARBfile


// ==================================================================
// ======================= Settings functions =======================
// ==================================================================

int Show_Settings ( void )
{	int iChannel, ich, i;


	LCDclear();
   if ( ChParams.Name[0] == NAME_FLAG ) // if name present
	{
   	sprintf ( LCDstring, "SaveSet %d name:\n", SaveSetNbr );
   	LCDprint ( "SaveSet name:\n" );
   	LCDprint ( &ChParams.Name[1] );
      MsDelay (2000);
	}

	costate
   {
	LCDclear();
	snprintf ( LCDstring, LCD_COLUMNS+1, "    PWM Settings" );
	LCDprint ( LCDstring );
	snprintf ( LCDstring, LCD_COLUMNS+1, "\nFreq = %ld", ChParams.Frequency[PWM_INDEX] );
	LCDprint ( LCDstring );

// show PWM values
	for ( iChannel=1; iChannel<=2; iChannel++ )
	{	ich = iChannel-1;						// 0 relative index for array access
   	snprintf ( LCDstring, LCD_COLUMNS+1, "\nCh%1d ", iChannel );
		LCDprint ( LCDstring );

		switch ( ChParams.PWM_Stop_Units[ich] )
      {
		case UNITS_STOP_DUTYCYCLE:       // 1234567890
         snprintf ( LCDstring, LCD_COLUMNS+1, "DC = %5.1f%%", ChParams.PWM_Stop_Value[ich] );
			LCDprint ( LCDstring );
			break;
		case UNITS_STOP_WIDTH:           // 1234567890
         snprintf ( LCDstring, LCD_COLUMNS+1, "PW = %7.1fus", ChParams.PWM_Stop_Value[ich] );
			LCDprint ( LCDstring );
			break;
		case UNITS_STOP_COUNT:           // 1234567890123
         snprintf ( LCDstring, LCD_COLUMNS+1, "Count = %4d", ChParams.PWM_Stop_Count[ich] );
			LCDprint ( LCDstring );
			break;
		case UNITS_STOP_PHASE:           // 1234567890123
         snprintf ( LCDstring, LCD_COLUMNS+1, "Phase = %5.1f\337", ChParams.PWM_Stop_Value[ich] );
			LCDprint ( LCDstring );
			break;
		}
   } //  each PWM channel

	GetSwitch_B ();

// Display PPM values

	for ( iChannel=1; iChannel<=4; iChannel++ )
	{	ich = iChannel-1;						// 0 relative index for array access

		LCDclear();
		sprintf ( LCDstring, "PPM Ch%1d Freq=%ld\n\r", iChannel, ChParams.Frequency[PPM_INDEX] );
		LCDprint ( LCDstring );
               // 1234567890123
      LCDprint ( "Start     Stop\n\r" );

// display start and stop modes
		switch ( ChParams.PPM_Start_Units[ich] )
      {
		case UNITS_START_COUNT:
         sprintf ( LCDstring, "Count" );
			break;
		case UNITS_START_PHASE:
         sprintf ( LCDstring, "Phase" );
			break;
		case UNITS_START_DELAY:
         sprintf ( LCDstring, "Delay" );
			break;
		}
		LCDprint ( LCDstring );

      LCDcursorPos ( 0, 11 );				// center of current line
		switch ( ChParams.PPM_Stop_Units[ich] )
      {
		case UNITS_STOP_COUNT:
         sprintf ( LCDstring, "Count\n\r" );
			break;
		case UNITS_STOP_PHASE:
         sprintf ( LCDstring, "Phase\n\r" );
			break;
		case UNITS_STOP_DELAY:
         sprintf ( LCDstring, "Delay\n\r" );
			break;
		case UNITS_STOP_WIDTH:
         sprintf ( LCDstring, "PulseWidth\n\r" );
			break;
		case UNITS_STOP_DUTYCYCLE:
         sprintf ( LCDstring, "DutyCycle\n\r" );
			break;
		}
		LCDprint ( LCDstring );

// display start and stop values

		switch ( ChParams.PPM_Start_Units[ich] )
      {
		case UNITS_START_COUNT:
         sprintf ( LCDstring, "%5d", ChParams.PPM_Start_Count[ich] );
			break;
		case UNITS_START_PHASE:
         sprintf ( LCDstring, "%5.1f\337", ChParams.PPM_Start_Value[ich] );
			break;
		case UNITS_START_DELAY:
         sprintf ( LCDstring, "%7.2fus", ChParams.PPM_Start_Value[ich] );
			break;
		}
		LCDprint ( LCDstring );

      LCDcursorPos ( 0, 11 );				// center of current line
		switch ( ChParams.PPM_Stop_Units[ich] )
      {
		case UNITS_STOP_COUNT:
         sprintf ( LCDstring, "%5d", ChParams.PPM_Stop_Count[ich] );
			break;
		case UNITS_STOP_PHASE:
         sprintf ( LCDstring, "%5.1f\337", ChParams.PPM_Stop_Value[ich] );
			break;
		case UNITS_STOP_DELAY:
         sprintf ( LCDstring, "%7.2fus", ChParams.PPM_Stop_Value[ich] );
			break;
		case UNITS_STOP_WIDTH:
         sprintf ( LCDstring, "%7.2fus", ChParams.PPM_Stop_Value[ich] );
			break;
		case UNITS_STOP_DUTYCYCLE:
         sprintf ( LCDstring, "%5.2f%%", ChParams.PPM_Stop_Value[ich] );
			break;
		}
		LCDprint ( LCDstring );

		i = GetSwitch_B ();
      if ( i==MENU_UP )
      {	if ( iChannel>1 ) iChannel -= 2;
      	else if ( iChannel==1 ) iChannel--;
      }

   } //  each PPM channel

// display DDS values
	LCDclear();
	LCDprint ( "    DDS Settings" );
	sprintf ( LCDstring, "\nFreq = %ld", ChParams.Frequency[DDS_INDEX] );
	LCDprint ( LCDstring );
	sprintf ( LCDstring, "\nAmplitude = %5.2f", ChParams.Amplitude[DDS_INDEX] );
	LCDprint ( LCDstring );
	sprintf ( LCDstring, "\nOffset = %5.2f", ChParams.Offset[DDS_INDEX] );
	LCDprint ( LCDstring );

	GetSwitch_B ();
   } // end costate

	return 1;
} // Show_Settings


/*
	1 <= Save Set Number <= MAX_SAVESET

RETURN VALUE:	 0 if successful
					-1 invalid address or range
               -2 No valid user block found (block version 3 or later)
 				   -3 flash writing error
*/

int Save_Settings ( void )
{	int i, offset, NameLen;
	long templong;

				 //123456789012345
	LCDprint ( "Save Set Nbr: " );
   templong = SaveSetNbr;					// default to current saveset
	GetLong_B ( &templong, 2, 15 );
   if ( templong < 1) templong = 1;
	if ( templong > MAX_SAVESET ) templong = MAX_SAVESET;
	offset = UB_OFFSET+(((int)templong-1)*CHPARAMS_SIZE);

   NameLen = sizeof(ChParams.Name);
   if ( templong != SaveSetNbr)			// if different save set
	   readUserBlock( &ChParams.Name, offset, NameLen ); // read its name

   if ( ChParams.Name[0] != NAME_FLAG )// if no name yet
   {	memset ( ChParams.Name, ' ', sizeof (ChParams.Name) );
   	ChParams.Name[0] = NAME_FLAG;		// insert flag byte
   	ChParams.Name[NameLen-1] = 0; 	// trailing NULL byte
   }
	LCDclear();
	LCDprint ( "Enter Save Set name\n\r" );
	GetText_B ( &ChParams.Name[1], MAX_NAME_LENGTH-2, 0 );
   							// leave room for leading flag and trailing NULL bytes

	ChParams.Version = VERSION;
   i = writeUserBlock( offset, &ChParams, CHPARAMS_SIZE );
	LCDcursorPos(4,1);
	switch (i)
   {
   case 0:
   	LCDprint ( "Success" );
      break;
   case -1:
   	LCDprint ( "Invalid addr/range" );
      break;
   case -2:
   	LCDprint ( "No valid user block" );
      break;
   case -3:
   	LCDprint ( "Flash write error" );
      break;
	}
   MsDelay ( 2000 );
   return 1;
} // Save_Settings

char	SaveSetnames[MAX_LIST_ITEMS][LIST_ITEM_SIZE];

int Read_Settings ( void )
{	int i, offset, NbrBytes, NbrSets;
	char flag;

  	LCDclear();
	NbrBytes = CHPARAMS_SIZE;

	NbrSets = 0;								// nbr of sets found
	if ( UserValue[2] )						// if auto-start
   {	SaveSetNbr = 1;						// default set number
   }
   else
   {
      while ( 1 )
      {
			offset = UB_OFFSET+(NbrBytes*NbrSets);
			i = readUserBlock( &ChParams, offset, NbrBytes );
			if ( !i  &&  (ChParams.Name[0] == NAME_FLAG) )
         {	memcpy ( &SaveSetnames[NbrSets], &ChParams.Name[1], LIST_ITEM_SIZE );
         	NbrSets++;
			}
         else break;
      }
   }
   if ( NbrSets )
   {	LCDprint ( "Select a Save Set:\n\r" );
		if (SaveSetNbr) ListItemState = 1 << (int)SaveSetNbr-1;
      else ListItemState = 0;
		i = GetListEntry_B ( (const char *)SaveSetnames, NbrSets, SINGLE_ITEM );
		SaveSetNbr = 1;
		while ( ListItemState )
      {	if ( ListItemState & 1 ) break;
			ListItemState >>= 1;
      	SaveSetNbr++;
      }
   }
	if ( SaveSetNbr > MAX_SAVESET ) SaveSetNbr = MAX_SAVESET;

	offset = UB_OFFSET+(((int)SaveSetNbr-1)*NbrBytes);

   sprintf ( LCDstring, "Reading Save Set %d\n\r", SaveSetNbr );
	LCDprint ( LCDstring );
   i = readUserBlock( &ChParams, offset, NbrBytes );
	if ( ChParams.Version != VERSION  &&  !i ) i = -3;

	if (ChParams.Name[0] == NAME_FLAG)
   {	LCDprint ( &ChParams.Name[1] );
		LCDline (LCDlineNbr+1);
   }

   if ( ChParams.DDS_Amp_Corr < 0.5  ||  ChParams.DDS_Amp_Corr > 2 )
   		ChParams.DDS_Amp_Corr = 1.0;	// valid value not written yet
   if ( ChParams.DDS_Off_Corr < 0.5  ||  ChParams.DDS_Off_Corr > 2 )
   		ChParams.DDS_Off_Corr = 1.0;	// valid value not written yet

	switch (i)
   {
   case 0:
   	LCDprint ( "Success" );
      Channel = 1;
      Use_Settings();
      break;
   case -1:
   	LCDprint ( "Invalid addr/range" );
      break;
   case -2:
   	LCDprint ( "No valid user block" );
      break;
   case -3:
   	LCDprint ( "Invalid Version" );
      break;
	}
   MsDelay ( 2000 );
   if ( UserValue[2] == 1 ) return i;
   return 1;
} // Read_Settings


#define INIT_FREQ	1000

int Set_Def_Settings ( void )
{	int i;

// initialize all the parameters
	memset ( &ChParams, 0, CHPARAMS_SIZE );

// start: set PWM values
	for ( i=0; i<4; i++ )
   {	ChParams.PWM_Stop_Units[i] = UNITS_STOP_COUNT;
   	ChParams.PWM_Stop_Count[i] = 512;
   	ChParams.PWM_Stop_Value[i] = ChParams.PWM_Stop_Count[i];
	}
	ChParams.Frequency[PWM_INDEX] = INIT_FREQ;
  	ChParams.PWMx4Frequency = 20000L;
// end: set PWM values

// start: set PPM values
	for ( i=0; i<4; i++ )					// set start and stop values
  	{	ChParams.PPM_Start_Units[i] = UNITS_START_COUNT;
     	ChParams.PPM_Start_Count[i] = 0;
     	ChParams.PPM_Start_Value[i] = ChParams.PPM_Start_Count[i];
		ChParams.PPM_Stop_Units[i] = UNITS_STOP_COUNT;
  	   ChParams.PPM_Stop_Count[i] = 5000;
  	   ChParams.PPM_Stop_Value[i] = ChParams.PPM_Stop_Count[i];
   }
	ChParams.Frequency[PPM_INDEX] = INIT_FREQ;
// end: set PPM values

// set DDS parameters
	ChParams.Frequency[DDS_INDEX] = 1000;
	ChParams.Amplitude[DDS_INDEX] = 1.0;
	ChParams.Offset[DDS_INDEX] = 0.0;
   ChParams.DDSPhase[0] = ChParams.DDSPhase[1] = 0;
   ChParams.DDS_Amp_Corr = 1.0;
   ChParams.DDS_Off_Corr = 1.0;
// end: set DDS parameters

// misc
	ChParams.CPU_CorrectionPPM = 0;
	ChParams.Version = VERSION;

	Use_Settings();

	return 1;
} // Set_Def_Settings


int Use_Settings ( void )
{	long Correction;
	unsigned offset;
   int i;

// get CPU frequency and compensate for measured error
   CPU_Frequency = get_cpu_frequency();
   Correction = ( (CPU_Frequency+500000L)/1000000L ) * ChParams.CPU_CorrectionPPM;
   CPU_Frequency -= Correction;
   fCPU_Frequency = (float)CPU_Frequency;

   PPM_LowFrequency = CPU_Frequency/(TAPRShadow+1)/65536L;
   PPM_LowFrequency += 1;					// a little margin

// set the DDS, PWM and PPM signals
	PWM_Set_Freq ( ChParams.Frequency[PWM_INDEX] );
	PPM_Set_Freq ( ChParams.Frequency[PPM_INDEX] );
	DDSsetFreq ( 0, ChParams.Frequency[DDS_INDEX], 0 );
   DDSAmplitude ( ChParams.Amplitude[DDS_INDEX] );
   DDSOffset ( ChParams.Offset[DDS_INDEX] );

   return(1);
} // Use_Settings

//	WARNING, Will Robinson!  WARNING!
// This function forces the calibration frequency to 1MHz in order to
//		calculate the Parts Per Million (PPM) error
int Set_Corr ( void )
{
	long CalcFreq, Frequency, Correction;

   CPU_Frequency = get_cpu_frequency();
   fCPU_Frequency = (float)CPU_Frequency;

	Frequency = 1000000L;					// set frequency to 1MHz
	CalcFreq = PPM_Set_Freq(Frequency);

	LCDprint ( "Enter Measured Freq.\n" );
				// 12345678901234567890
            //  +12345678Hz
	LCDprint ( "          Hz"  );
	while ( !GetLong(&Frequency,8,2) );	// get 1MHz reading

// since the programmed frequency is 1Mhz, the calculated error is in PPM
	Correction = CalcFreq - Frequency;	// calculate PPM error
	ChParams.CPU_CorrectionPPM = (int)Correction; // save it

// actual error = CPU_Frequency in MHz * Correction
   Correction = ( (CPU_Frequency+500000L)/1000000L ) * ChParams.CPU_CorrectionPPM;
  		// the +500000 is to ensure rounding rather than the default of truncation
   CPU_Frequency -= Correction;
   fCPU_Frequency = (float)CPU_Frequency;
	CalcFreq = PPM_Set_Freq(Frequency);
   return 1;

} // Set_Corr


// ==================================================================
// ======================== Utility functions =======================
// ==================================================================


#asm
; sp+6: data value
; sp+4: shadow register address
; sp+2: I/O address (also in HL)
; sp+0: return address
WrPortI2::		; ( int Address, int *ShadowRegister, int DataValue );
		ld		ix, HL				; get I/O address
   	ld		iy, (SP+4)			; get shadow register address
   	ld		HL, (SP+6)			; get data value
		ld		(iy), HL				; save in shadow register
ioi	ld		(ix), HL
		ret

#endasm

void KnightRider ( void )
{	int i, delay;

	delay = 100;

	for ( i=0; i<=4; i++ )
	{	LEDS (1);
  		MsDelay (delay);
		LEDS (2);
  		MsDelay (delay);
		LEDS (4);
  		MsDelay (delay);
		LEDS (8);
  		MsDelay (delay);
		LEDS (4);
  		MsDelay (delay);
		LEDS (2);
  		MsDelay (delay);
   }
	LEDS (1);
	MsDelay (delay);
	LEDS (0);
}

